@@ -63,17 +63,25 @@ def check_valid_batch_dict(in_dict: dict, no_nan=False) -> bool:
     bool
         True, if it passes the checks.
     """
-    assert len(in_dict) > 1
+    assert len(in_dict) > 1, "At least one valid batch is required."
     batch1 = in_dict[list(in_dict.keys())[0]]
     base_columns = set(batch1.columns)
     check = True
-    for _, batch in in_dict.items():
+    for bid, batch in in_dict.items():
         # Check 1
         check = check & (base_columns == set(batch.columns))
+        if check is False:
+            assert False, "The column names must be the same in all batches. Differs in {bid}."
+
         # Check 2
         check *= batch.select_dtypes(include=[np.number]).shape[1] == batch.shape[1]
+        if check is False:
+            assert False, "All columns must be a numeric type. Differs in {bid}."
+
         # Check 3
         check *= batch.isna().values.sum() == 0
+        if check is False:
+            assert False, "No missing values allowed. Missing values found in {bid}."
 
     return check
 
@@ -90,9 +98,7 @@ def dict_to_melted(
         if idx == 0:
             num_rows = batch.shape[0]
             sequence = np.arange(0, num_rows)
-        assert (
-            num_rows == batch.shape[0]
-        ), "All batches must have the same number of samples"
+        assert num_rows == batch.shape[0], "All batches must have the same number of samples"
 
         if insert_batch_id_column and batch_id_col not in batch:
             batch.insert(0, batch_id_col, batch_id)
@@ -105,20 +111,18 @@ def dict_to_melted(
     return out_df
 
 
-def dict_to_wide(in_df: dict, group_by_time=False) -> pd.DataFrame:
+def dict_to_wide(in_df: dict, group_by_batch=False) -> pd.DataFrame:
     """
     Data must be aligned (warped) already so that every batch has the same number of *rows*!
 
-    `group_by_time`: means that all the data from the first batch is on the left of the output
-    dataframe, and the last batch is collected on the right.
+    `group_by_batch`, if True, means that all the data from the first batch is on the left
+    of the output dataframe, and the last batch is collected on the right.
 
-    If `group_by_time` is False, then all data for the same tag are grouped together, side-by-side.
+    If `group_by_batch` is False, then data for the same tag are grouped together, side-by-side.
     """
-    out_df = dict_to_melted(
-        in_df=in_df, insert_batch_id_column=True, insert_sequence_column=True
-    )
+    out_df = dict_to_melted(in_df=in_df, insert_batch_id_column=True, insert_sequence_column=True)
     aligned_wide_df = out_df.pivot(index="batch_id", columns="__sequence__")
-    if group_by_time:
+    if group_by_batch:
         pass
         # TODO: use the hierarchical indexing and regroup the columns
 
@@ -158,9 +162,7 @@ def wide_to_dict():
     pass
 
 
-def melt_df_to_series(
-    in_df: pd.DataFrame, exclude_columns=["batch_id"], name=None
-) -> pd.Series:
+def melt_df_to_series(in_df: pd.DataFrame, exclude_columns=["batch_id"], name=None) -> pd.Series:
     """Returns a Series with a multilevel-index, melted from the DataFrame"""
     out = in_df.drop(exclude_columns, axis=1).T.stack()
     out.name = name
