@@ -1,7 +1,7 @@
 # Built-in libraries
 import math
 import random
-from typing import Optional
+from typing import Any, Dict, Optional
 
 # Plotting settings
 import plotly.graph_objects as go
@@ -51,14 +51,17 @@ def plot_to_HTML(filename: str, fig: dict):
 def plot__all_batches_per_tag(
     df_dict: dict,
     tag: str,
+    tag_y2: str = None,
     time_column: str = None,
     extra_info="",
     batches_to_highlight={},
     x_axis_label: str = "Time [sequence order]",
+    highlight_width: int = 5,
     html_image_height: int = 900,
     html_aspect_ratio_w_over_h: float = 16 / 9,
-    y_limits: tuple = (None, None),
-) -> dict:
+    y1_limits: tuple = (None, None),
+    y2_limits: tuple = (None, None),
+) -> go.Figure:
     """Plots a particular `tag` over all batches in the given dataframe `df`.
 
     Parameters
@@ -66,7 +69,11 @@ def plot__all_batches_per_tag(
     df_dict : dict
         Standard data format for batches.
     tag : str
-        Which tag to plot?
+        Which tag to plot? [on the y1 (left) axis]
+    tag_y2 : str, optional
+        Which tag to plot? [on the y2 (right) axis]
+        Tag will be plotted with different scaling on the secondary axis, to allow time-series
+        comparisons to be easier.
     time_column : str, optional
         Which tag on the x-axis. If not specified, creates sequential integers, starting from 0
         if left as the default, `None`.
@@ -78,16 +85,27 @@ def plot__all_batches_per_tag(
         The highlighted batches will be shown with a heavier line.
     x_axis_label : str, optional
         String label for the x-axis, by default "Time [sequence order]"
-
+    highlight_width: int, optional
+        The width of the highlighted lines; default = 5.
     html_image_height : int, optional
         HTML image output height, by default 900
     html_aspect_ratio_w_over_h : float, optional
         HTML image aspect ratio: 16/9 (therefore the default width will be 1600 px)
+    y1_limits: tuple, optional
+        Axis limits enforced on the y1 (left) axis. Default is (None, None) which means the data
+        themselves are used to determine the limits. Specify BOTH limits. Plotly requires
+        (at the moment https://github.com/plotly/plotly.js/issues/400) that you specify both.
+        Order: (low limit, high limit)
+    y2_limits: tuple, optional
+        Axis limits enforced on the y2 (right) axis. Default is (None, None) which means the data
+        themselves are used to determine the limits. Specify BOTH limits. Plotly requires
+        (at the moment https://github.com/plotly/plotly.js/issues/400) that you specify both.
+
 
     Returns
     -------
-    dict
-        Standard Plotly dict with two keys: `data` and `layout`.
+    go.Figure
+        Standard Plotly fig object (dictionary-like).
     """
     unique_items = list(df_dict.keys())
     n_colours = len(unique_items)
@@ -97,51 +115,108 @@ def plot__all_batches_per_tag(
     colours = [get_rgba_from_triplet(c, as_string=True) for c in colours]
     colour_assignment = dict(zip(unique_items, colours))
 
-    traces = []
-    highlight_traces = []
-    highlight_width = 5
+    # traces = []
+    # highlight_traces = []
     regular_style = dict()
-
     highlight_dict = {}
     for key, val in batches_to_highlight.items():
         highlight_dict.update({item: key for item in val if item in df_dict.keys()})
 
+    fig = go.Figure()
+
     for batch_name, batch_df in df_dict.items():
         assert (
             tag in batch_df.columns
         ), f"Tag '{tag}' not found in the batch with id {batch_name}."
+        if tag_y2:
+            assert (
+                tag_y2 in batch_df.columns
+            ), f"Tag '{tag}' not found in the batch with id {batch_name}."
         if time_column in batch_df.columns:
             time_data = batch_df[time_column]
         else:
             time_data = list(range(batch_df.shape[0]))
 
         if batch_name in highlight_dict.keys():
-            trace = go.Scatter(
-                x=time_data,
-                y=batch_df[tag],
-                line=dict(width=highlight_width, color=highlight_dict[batch_name]),
-                name=batch_name,
-                mode="lines",
-                opacity=0.8,
-            )
-            highlight_traces.append(trace)
+            continue  # come to this later
         else:
             regular_style["color"] = colour_assignment[batch_name]
-            trace = go.Scatter(
-                x=time_data,
-                y=batch_df[tag],
-                name=batch_name,
-                line=regular_style,
-                mode="lines",
-                opacity=0.8,
+            fig.add_trace(
+                go.Scatter(
+                    x=time_data,
+                    y=batch_df[tag],
+                    name=batch_name,
+                    line=regular_style,
+                    mode="lines",
+                    opacity=0.8,
+                    yaxis="y1",
+                )
             )
-            traces.append(trace)
+            if tag_y2:
+                fig.add_trace(
+                    go.Scatter(
+                        x=time_data,
+                        y=batch_df[tag_y2],
+                        name=batch_name,
+                        line=regular_style,
+                        mode="lines",
+                        opacity=0.8,
+                        yaxis="y2",
+                    )
+                )
 
-    # Add the highlighted one last
-    traces.extend(highlight_traces)
+    # Add the highlighted batches last: therefore, sadly, we have to do another run-through.
+    # Plotly does not yet support z-orders.
+    for batch_name, batch_df in df_dict.items():
+        if time_column in batch_df.columns:
+            time_data = batch_df[time_column]
+        else:
+            time_data = list(range(batch_df.shape[0]))
 
-    layout = go.Layout(
-        title=f"For all batches: '{tag}'."
+        if batch_name in highlight_dict.keys():
+            fig.add_trace(
+                go.Scatter(
+                    x=time_data,
+                    y=batch_df[tag],
+                    line=dict(width=highlight_width, color=highlight_dict[batch_name]),
+                    name=batch_name,
+                    mode="lines",
+                    opacity=0.8,
+                    yaxis="y1",
+                )
+            )
+            if tag_y2:
+                fig.add_trace(
+                    go.Scatter(
+                        x=time_data,
+                        y=batch_df[tag_y2],
+                        line=dict(
+                            width=highlight_width, color=highlight_dict[batch_name]
+                        ),
+                        name=batch_name,
+                        mode="lines",
+                        opacity=0.8,
+                        yaxis="y2",
+                    )
+                )
+
+    yaxis1_dict = dict(
+        title=tag, gridwidth=2, matches="y1", showticklabels=True, side="left"
+    )
+    if (y1_limits[0] is not None) or (y1_limits[1] is not None):
+        yaxis1_dict["autorange"] = False
+        yaxis1_dict["range"] = y1_limits
+
+    yaxis2_dict: Dict[str, Any] = dict(
+        title=tag_y2, gridwidth=2, matches="y2", showticklabels=True, side="right"
+    )
+    if (y2_limits[0] is not None) or (y2_limits[1] is not None):
+        yaxis2_dict["autorange"] = False
+        yaxis2_dict["range"] = y2_limits
+
+    fig.update_layout(
+        title=f"Plot of: '{tag}'"
+        + (f" on left axis; with '{str(tag_y2)}' on right axis." if tag_y2 else ".")
         + (f" [{str(extra_info)}]" if extra_info else ""),
         hovermode="closest",
         showlegend=True,
@@ -154,11 +229,13 @@ def plot__all_batches_per_tag(
         ),
         autosize=False,
         xaxis=dict(title=x_axis_label, gridwidth=1),
-        yaxis=dict(title=tag, gridwidth=2),
+        yaxis=yaxis1_dict,
         width=html_aspect_ratio_w_over_h * html_image_height,
         height=html_image_height,
     )
-    return dict(data=traces, layout=layout)
+    if tag_y2:
+        fig.update_layout(yaxis2=yaxis2_dict)
+    return fig
 
 
 def plot__tag_time(
